@@ -50,6 +50,10 @@ class RuntimeConfig:
             'db_paths': {
                 'audit_db': './data/audit_logs.db',
                 'auth_db': './data/auth.db'
+            },
+            'env_presets': {
+                'auto_apply_enabled': True,  # Auto-apply preset on startup
+                'active_preset_id': None     # ID of preset to auto-apply (null = none)
             }
         }
     
