@@ -631,3 +631,122 @@ class EnvPresetManager:
             
             row = cursor.fetchone()
             return row[0] if row else None
+    
+    def apply_preset(self, preset_id: int) -> Dict[str, Any]:
+        """
+        Apply preset variables to current process environment (os.environ).
+        
+        Variables are set in the current process and will be available to:
+        - Code executed by code_executor
+        - Child processes spawned by this instance
+        - Any code that reads os.environ or os.getenv()
+        
+        Variables persist only for the lifetime of the current process.
+        To survive restarts, use auto_apply_env_preset_id in runtime_config.json.
+        
+        Args:
+            preset_id: The preset ID to apply
+            
+        Returns:
+            dict: Applied variables count, preset name, and variable keys
+            
+        Raises:
+            ValueError: If preset not found
+        """
+        preset = self.get_preset(preset_id, mask_values=False)
+        if not preset:
+            raise ValueError(f"Preset {preset_id} not found")
+        
+        applied_count = 0
+        applied_vars = []
+        
+        for var in preset['variables']:
+            key = var['key']
+            value = var['value']
+            
+            # Set in current process environment
+            os.environ[key] = value
+            applied_count += 1
+            applied_vars.append(key)
+            
+            logger.debug(f"Set environment variable: {key}")
+        
+        logger.info(
+            f"Applied {applied_count} variables from preset "
+            f"'{preset['preset_name']}' (ID: {preset_id})"
+        )
+        
+        return {
+            'preset_id': preset_id,
+            'preset_name': preset['preset_name'],
+            'applied_count': applied_count,
+            'variables': applied_vars
+        }
+    
+    def remove_preset_variables(self, preset_id: int) -> Dict[str, Any]:
+        """
+        Remove variables from current process environment (os.environ).
+        
+        Only removes variables that are currently set and match the preset.
+        Reserved system variables are never removed.
+        
+        Args:
+            preset_id: The preset ID whose variables should be removed
+            
+        Returns:
+            dict: Removed variables count and variable keys
+            
+        Raises:
+            ValueError: If preset not found
+        """
+        preset = self.get_preset(preset_id, mask_values=False)
+        if not preset:
+            raise ValueError(f"Preset {preset_id} not found")
+        
+        removed_count = 0
+        removed_vars = []
+        
+        for var in preset['variables']:
+            key = var['key']
+            
+            # Skip reserved keys (safety check)
+            if key.upper() in self.RESERVED_KEYS:
+                logger.warning(f"Skipped removing reserved variable: {key}")
+                continue
+            
+            # Remove if exists
+            if key in os.environ:
+                del os.environ[key]
+                removed_count += 1
+                removed_vars.append(key)
+                logger.debug(f"Removed environment variable: {key}")
+        
+        logger.info(
+            f"Removed {removed_count} variables from preset "
+            f"'{preset['preset_name']}' (ID: {preset_id})"
+        )
+        
+        return {
+            'preset_id': preset_id,
+            'preset_name': preset['preset_name'],
+            'removed_count': removed_count,
+            'variables': removed_vars
+        }
+    
+    def get_active_environment_snapshot(self) -> Dict[str, str]:
+        """
+        Get a snapshot of current environment variables.
+        
+        Useful for debugging and verifying which variables are currently set.
+        Excludes reserved system variables for security.
+        
+        Returns:
+            dict: Current environment variables (non-reserved only)
+        """
+        snapshot = {}
+        for key, value in os.environ.items():
+            # Exclude reserved system variables from snapshot
+            if key.upper() not in self.RESERVED_KEYS:
+                snapshot[key] = value
+        
+        return snapshot
