@@ -12,7 +12,7 @@ from pathlib import Path
 from ..utils.audit_logger import AuditLogger
 from ..utils.env_manager import EnvPresetManager
 from .auth import AuthManager
-from ..config.runtime_config import get_web_config, save_web_config
+from ..config.runtime_config import get_web_config, save_web_config, RuntimeConfig
 
 logger = logging.getLogger(__name__)
 
@@ -37,6 +37,62 @@ def _save_web_config(config: dict) -> bool:
         return False
 
 
+def _auto_apply_env_preset(env_manager: EnvPresetManager):
+    """
+    Auto-apply ALL environment presets on application startup.
+    
+    Applies all stored presets to os.environ, ensuring environment variables
+    survive server restarts. Variables from later presets override earlier ones
+    if there are conflicts.
+    
+    This ensures environment variables survive server restarts.
+    """
+    try:
+        runtime_config = RuntimeConfig()
+        
+        auto_apply_enabled = runtime_config.get('env_presets.auto_apply_enabled', True)
+        if not auto_apply_enabled:
+            logger.info("Environment preset auto-apply is disabled")
+            return
+        
+        # Get all presets
+        presets = env_manager.get_presets(include_var_count=False)
+        
+        if not presets:
+            logger.info("No environment presets configured for auto-apply")
+            return
+        
+        # Apply each preset
+        total_applied = 0
+        all_variables = []
+        applied_presets = []
+        
+        for preset in presets:
+            try:
+                result = env_manager.apply_preset(preset['id'])
+                total_applied += result['applied_count']
+                all_variables.extend(result['variables'])
+                applied_presets.append(f"{preset['preset_name']} ({result['applied_count']} vars)")
+            except Exception as e:
+                logger.error(f"Failed to apply preset '{preset['preset_name']}': {e}")
+        
+        if total_applied > 0:
+            logger.info("="*60)
+            logger.info("🔧 AUTO-APPLIED ENVIRONMENT PRESETS")
+            logger.info(f"  Presets Applied: {len(applied_presets)}")
+            for preset_info in applied_presets:
+                logger.info(f"    • {preset_info}")
+            logger.info(f"  Total Variables: {total_applied}")
+            logger.info(f"  Keys: {', '.join(set(all_variables))}")
+            logger.info("  All executed code will have access to these variables")
+            logger.info("="*60)
+        
+    except Exception as e:
+        logger.error(f"Error during auto-apply: {e}")
+        import traceback
+        logger.debug(traceback.format_exc())
+
+
 def create_app(config: dict) -> Flask:
     """
     Create and configure Flask application.
@@ -73,6 +129,9 @@ def create_app(config: dict) -> Flask:
         secret_key=config.get('SECRET_KEY', '')
     )
     
+    # Auto-apply active environment preset on startup (if configured)
+    _auto_apply_env_preset(env_manager)
+    
     # Store in app config for access in routes
     app.config['audit_logger'] = audit_logger
     app.config['auth_manager'] = auth_manager
@@ -394,7 +453,7 @@ def create_app(config: dict) -> Flask:
     @app.route('/api/env/presets', methods=['POST'])
     @login_required
     def api_create_env_preset():
-        """Create a new environment preset."""
+        """Create a new environment preset and optionally apply it."""
         try:
             env_manager = app.config.get('env_manager')
             if not env_manager:
@@ -405,6 +464,7 @@ def create_app(config: dict) -> Flask:
             project_id = data.get('project_id')
             description = data.get('description', '').strip()
             variables = data.get('variables', [])
+            auto_apply = data.get('auto_apply', True)  # Apply by default
             
             if not preset_name:
                 return jsonify({'error': 'Preset name is required'}), 400
@@ -425,7 +485,24 @@ def create_app(config: dict) -> Flask:
                 variables=variables
             )
             
-            return jsonify({'success': True, 'preset_id': preset_id})
+            # Auto-apply after saving
+            apply_result = None
+            if auto_apply:
+                try:
+                    apply_result = env_manager.apply_preset(preset_id)
+                    logger.info(
+                        f"User '{request.username}' created and applied preset '{preset_name}' "
+                        f"with {apply_result['applied_count']} variables"
+                    )
+                except Exception as e:
+                    logger.warning(f"Created preset but failed to apply: {e}")
+            
+            return jsonify({
+                'success': True,
+                'preset_id': preset_id,
+                'applied': apply_result is not None,
+                'applied_count': apply_result['applied_count'] if apply_result else 0
+            })
         except ValueError as e:
             return jsonify({'error': str(e)}), 400
         except Exception as e:
@@ -454,13 +531,21 @@ def create_app(config: dict) -> Flask:
     @app.route('/api/env/presets/<int:preset_id>', methods=['PUT'])
     @login_required
     def api_update_env_preset(preset_id):
-        """Update preset metadata or variables."""
+        """Update preset metadata or variables, and apply changes."""
         try:
             env_manager = app.config.get('env_manager')
             if not env_manager:
                 return jsonify({'error': 'Environment presets not initialized'}), 500
             
             data = request.get_json()
+            auto_apply = data.get('auto_apply', True)  # Apply by default
+            
+            # Get current variables before update (to detect deletions)
+            old_preset = env_manager.get_preset(preset_id, mask_values=False)
+            if not old_preset:
+                return jsonify({'error': 'Preset not found'}), 404
+            
+            old_keys = {var['key'] for var in old_preset['variables']}
             
             # Check if updating variables or metadata
             if 'variables' in data:
@@ -476,8 +561,45 @@ def create_app(config: dict) -> Flask:
                             return jsonify({'error': error, 'invalid_key': key}), 400
                 
                 env_manager.update_variables(preset_id, variables)
+                
+                # Auto-apply after updating
+                if auto_apply:
+                    try:
+                        # Get new variables after update
+                        new_preset = env_manager.get_preset(preset_id, mask_values=False)
+                        new_keys = {var['key'] for var in new_preset['variables']}
+                        
+                        # Remove deleted variables from environment
+                        deleted_keys = old_keys - new_keys
+                        for key in deleted_keys:
+                            if key in os.environ:
+                                del os.environ[key]
+                                logger.debug(f"Unset deleted variable: {key}")
+                        
+                        # Apply the updated preset (adds/updates variables)
+                        apply_result = env_manager.apply_preset(preset_id)
+                        
+                        logger.info(
+                            f"User '{request.username}' updated and applied preset "
+                            f"(applied: {apply_result['applied_count']}, removed: {len(deleted_keys)})"
+                        )
+                        
+                        return jsonify({
+                            'success': True,
+                            'applied': True,
+                            'applied_count': apply_result['applied_count'],
+                            'removed_count': len(deleted_keys),
+                            'removed_keys': list(deleted_keys)
+                        })
+                    except Exception as e:
+                        logger.warning(f"Updated preset but failed to apply: {e}")
+                        return jsonify({
+                            'success': True,
+                            'applied': False,
+                            'error': f"Saved but apply failed: {str(e)}"
+                        })
             else:
-                # Update metadata
+                # Update metadata only
                 preset_name = data.get('preset_name')
                 project_id = data.get('project_id')
                 description = data.get('description')
@@ -619,6 +741,139 @@ def create_app(config: dict) -> Flask:
             logger.error(f"Error validating key: {e}")
             return jsonify({'error': str(e)}), 500
     
+    @app.route('/api/env/presets/<int:preset_id>/apply', methods=['POST'])
+    @login_required
+    def api_apply_env_preset(preset_id):
+        """
+        Apply preset variables to current process environment (os.environ).
+        
+        Variables will be available to:
+        - Code executed by code_executor
+        - Any subprocess spawned by this instance
+        - Any code that reads os.environ or os.getenv()
+        
+        Variables persist for the lifetime of the current process.
+        """
+        try:
+            env_manager = app.config.get('env_manager')
+            if not env_manager:
+                return jsonify({'error': 'Environment presets not initialized'}), 500
+            
+            result = env_manager.apply_preset(preset_id)
+            
+            logger.info(
+                f"User '{request.username}' applied preset '{result['preset_name']}' "
+                f"with {result['applied_count']} variables"
+            )
+            
+            return jsonify({
+                'success': True,
+                'message': f"Applied {result['applied_count']} environment variables",
+                **result
+            })
+        except ValueError as e:
+            return jsonify({'error': str(e)}), 404
+        except Exception as e:
+            logger.error(f"Error applying preset {preset_id}: {e}")
+            return jsonify({'error': str(e)}), 500
+    
+    @app.route('/api/env/presets/<int:preset_id>/remove', methods=['POST'])
+    @login_required
+    def api_remove_env_preset(preset_id):
+        """
+        Remove preset variables from current process environment (os.environ).
+        
+        Only removes variables that are currently set.
+        Reserved system variables are never removed.
+        """
+        try:
+            env_manager = app.config.get('env_manager')
+            if not env_manager:
+                return jsonify({'error': 'Environment presets not initialized'}), 500
+            
+            result = env_manager.remove_preset_variables(preset_id)
+            
+            logger.info(
+                f"User '{request.username}' removed preset '{result['preset_name']}' "
+                f"with {result['removed_count']} variables"
+            )
+            
+            return jsonify({
+                'success': True,
+                'message': f"Removed {result['removed_count']} environment variables",
+                **result
+            })
+        except ValueError as e:
+            return jsonify({'error': str(e)}), 404
+        except Exception as e:
+            logger.error(f"Error removing preset {preset_id}: {e}")
+            return jsonify({'error': str(e)}), 500
+    
+    @app.route('/api/env/snapshot', methods=['GET'])
+    @login_required
+    def api_get_env_snapshot():
+        """
+        Get snapshot of current environment variables.
+        
+        Useful for debugging and verifying which variables are currently set.
+        Excludes reserved system variables.
+        """
+        try:
+            env_manager = app.config.get('env_manager')
+            if not env_manager:
+                return jsonify({'error': 'Environment presets not initialized'}), 500
+            
+            snapshot = env_manager.get_active_environment_snapshot()
+            
+            return jsonify({
+                'success': True,
+                'count': len(snapshot),
+                'variables': snapshot
+            })
+        except Exception as e:
+            logger.error(f"Error getting environment snapshot: {e}")
+            return jsonify({'error': str(e)}), 500
+    
+    @app.route('/api/env/auto-apply', methods=['GET'])
+    @login_required
+    def api_get_auto_apply_config():
+        """Get auto-apply configuration."""
+        try:
+            runtime_config = RuntimeConfig()
+            
+            return jsonify({
+                'success': True,
+                'auto_apply_enabled': runtime_config.get('env_presets.auto_apply_enabled', True)
+            })
+        except Exception as e:
+            logger.error(f"Error getting auto-apply config: {e}")
+            return jsonify({'error': str(e)}), 500
+    
+    @app.route('/api/env/auto-apply', methods=['POST'])
+    @login_required
+    def api_set_auto_apply_config():
+        """Set auto-apply configuration for ALL presets."""
+        try:
+            runtime_config = RuntimeConfig()
+            data = request.get_json()
+            
+            if 'auto_apply_enabled' in data:
+                runtime_config.set('env_presets.auto_apply_enabled', bool(data['auto_apply_enabled']))
+            
+            logger.info(
+                f"User '{request.username}' updated auto-apply config: "
+                f"enabled={data.get('auto_apply_enabled')} (will apply ALL presets on startup)"
+            )
+            
+            return jsonify({
+                'success': True,
+                'message': 'Auto-apply configuration saved. All presets will be applied on next server restart.',
+                'auto_apply_enabled': runtime_config.get('env_presets.auto_apply_enabled', True)
+            })
+        except Exception as e:
+            logger.error(f"Error setting auto-apply config: {e}")
+            return jsonify({'error': str(e)}), 500
+    
     # ===== End Environment Presets API =====
     
     # Security: Reject all undefined routes
